==> MRAConnector-0.91.tar.bz2: MRAConnector-0.91/core/Smdn.Net.Imap4.Client/Smdn.Net.Imap4.Client/ImapMessageInfo.OperationsStore.cs <==
// 
// Author:
//       smdn <smdn@smdn.jp>
// 
// Copyright (c) 2008-2014 smdn
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;

#if NET_3_5
using System.Linq;
#else
using Smdn.Collections;
#endif
using Smdn.Net.Imap4.Client.Session;

namespace Smdn.Net.Imap4.Client {
  partial class ImapMessageInfo {
    public void Refresh()
    {
      EnsureDynamicAttributesFetched(true);
    }

    private bool FindBySequence(ImapMessageAttributeBase attribute)
    {
      return attribute.Sequence == Sequence;
    }

    internal static readonly ImapFetchDataItem FetchStaticDataItem =
      ImapFetchDataItem.BodyStructure +
      ImapFetchDataItem.Envelope + 
      ImapFetchDataItem.InternalDate +
      ImapFetchDataItem.Rfc822Size;

    private void EnsureStaticAttributesFetched()
    {
      if (staticAttribute != null)
        return;

      CheckMessageNumberAvailable(SequenceOrUidSet);

      ImapMessageStaticAttribute[] staticAttrs;

      Mailbox.ProcessMessageStatusChangedResult(Client.Session.Fetch(SequenceOrUidSet,
                                                                     FetchStaticDataItem,
                                                                     out staticAttrs),
                                                staticAttrs);

      var staticAttr = staticAttrs.FirstOrDefault(FindBySequence);

      if (staticAttr == null) {
        SetDeleted();
        throw new ImapMessageVanishedException(this);
      }
      else {
        staticAttribute = staticAttr;
      }
    }

    internal static ImapFetchDataItem GetFetchDynamicDataItem(ImapOpenedMailboxInfo mailbox)
    {
      if (mailbox.IsModSequencesAvailable)
        return ImapFetchDataItem.Flags + ImapFetchDataItem.ModSeq;
      else
        return ImapFetchDataItem.Flags;
    }

    internal static ImapFetchDataItem TranslateFetchOption(ImapOpenedMailboxInfo mailbox,
                                                           ImapMessageFetchAttributeOptions options,
                                                           out bool fetchStaticAttr,
                                                           out bool fetchDynamicAttr)
    {
      fetchStaticAttr = (int)(options & ImapMessageFetchAttributeOptions.StaticAttributes) != 0;
      fetchDynamicAttr = (int)(options & ImapMessageFetchAttributeOptions.DynamicAttributes) != 0;

      var fetchDataItem = ImapFetchDataItem.Uid;

      if (fetchStaticAttr)
        fetchDataItem += ImapMessageInfo.FetchStaticDataItem;
      if (fetchDynamicAttr)
        fetchDataItem += ImapMessageInfo.GetFetchDynamicDataItem(mailbox);

      return fetchDataItem;
    }

    private void EnsureDynamicAttributesFetched(bool refresh)
    {
      if (!refresh && dynamicAttribute != null)
        return;

      CheckMessageNumberAvailable(SequenceOrUidSet);

      ImapMessageDynamicAttribute[] dynamicAttrs;

      Mailbox.ProcessMessageStatusChangedResult(Client.Session.Fetch(SequenceOrUidSet,
                                                                     GetFetchDynamicDataItem(Mailbox),
                                                                     out dynamicAttrs),
                                                dynamicAttrs);

      var dynamicAttr = dynamicAttrs.FirstOrDefault(FindBySequence);

      if (dynamicAttr == null) {
        SetDeleted();
        throw new ImapMessageVanishedException(this);
      }
      else {
        dynamicAttribute = dynamicAttr;
      }
    }

    public void ToggleFlags(ImapMessageFlag flag,
                            params ImapMessageFlag[] flags)
    {
      ToggleFlags(new ImapMessageFlagSet(flags.Prepend(flag)));
    }

    public void ToggleKeywords(string keyword,
                               params string[] keywords)
    {
      ToggleFlags(new ImapMessageFlagSet(keywords.Prepend(keyword)));
    }

    public void ToggleFlags(IImapMessageFlagSet flagsAndKeywords)
    {
      if (flagsAndKeywords == null)
        throw new ArgumentNullException("flagsAndKeywords");

      var addFlags = new ImapMessageFlagSet();
      var removeFlags = new ImapMessageFlagSet();

      EnsureDynamicAttributesFetched(false);

      foreach (var flag in flagsAndKeywords) {
        if (dynamicAttribute.Flags.Contains(flag))
          removeFlags.Add(flag);
        else
          addFlags.Add(flag);
      }

      if (0 < removeFlags.Count)
        Store(ImapStoreDataItem.RemoveFlags(removeFlags));

      if (0 < addFlags.Count)
        Store(ImapStoreDataItem.AddFlags(addFlags));
    }
  }
}

==> MRAConnector-1.12.tar.bz2: MRAConnector-1.12/core/Smdn.Net.Imap4.Client/Smdn.Net.Imap4.Client/ImapMessageInfo.OperationsStore.cs <==
// 
// Author:
//       smdn <smdn@smdn.jp>
// 
// Copyright (c) 2008-2014 smdn
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

#if NET_4_0
#define ASYNC_OPS
#endif

using System;
using System.Collections.Generic;
using System.Linq;
#if ASYNC_OPS
using System.Threading;
using System.Threading.Tasks;
#endif

using Smdn.Net.Imap4.Client.Session;

namespace Smdn.Net.Imap4.Client {
  partial class ImapMessageInfo {
    public void Refresh()
    {
      Client.RunExclusively(() => FetchDynamicAttributes());
    }

#if ASYNC_OPS
    public Task RefreshAsync()
    {
      return Client.StartTaskExclusively(() => FetchDynamicAttributes(),
                                         CancellationToken.None);
    }
#endif

    private bool FindBySequence(ImapMessageAttributeBase attribute)
    {
      return attribute.Sequence == Sequence;
    }

    internal static readonly ImapFetchDataItem FetchStaticDataItem =
      ImapFetchDataItem.BodyStructure +
      ImapFetchDataItem.Envelope + 
      ImapFetchDataItem.InternalDate +
      ImapFetchDataItem.Rfc822Size;

    private void EnsureStaticAttributesFetched()
    {
      if (staticAttribute != null)
        return;

      Client.RunExclusively(() => {
        CheckMessageNumberAvailable(SequenceOrUidSet);

        ImapMessageStaticAttribute[] staticAttrs;

        Mailbox.ProcessMessageStatusChangedResult(Client.Session.Fetch(SequenceOrUidSet,
                                                                       FetchStaticDataItem,
                                                                       out staticAttrs),
                                                  staticAttrs);

        var staticAttr = staticAttrs.FirstOrDefault(FindBySequence);

        if (staticAttr == null) {
          SetDeleted();
          throw new ImapMessageVanishedException(this);
        }
        else {
          staticAttribute = staticAttr;
        }
      });
    }

    private static readonly ImapFetchDataItem fetchFlagsAndModSeq = ImapFetchDataItem.Flags + ImapFetchDataItem.ModSeq;

    internal static ImapFetchDataItem GetFetchDynamicDataItem(ImapOpenedMailboxInfo mailbox)
    {
      if (mailbox.IsModSequencesAvailable)
        return fetchFlagsAndModSeq;
      else
        return ImapFetchDataItem.Flags;
    }

    internal static ImapFetchDataItem TranslateFetchOption(ImapOpenedMailboxInfo mailbox,
                                                           ImapMessageFetchAttributeOptions options,
                                                           out bool fetchStaticAttr,
                                                           out bool fetchDynamicAttr)
    {
      fetchStaticAttr = (int)(options & ImapMessageFetchAttributeOptions.StaticAttributes) != 0;
      fetchDynamicAttr = (int)(options & ImapMessageFetchAttributeOptions.DynamicAttributes) != 0;

      var fetchDataItem = ImapFetchDataItem.Uid;

      if (fetchStaticAttr)
        fetchDataItem += ImapMessageInfo.FetchStaticDataItem;
      if (fetchDynamicAttr)
        fetchDataItem += ImapMessageInfo.GetFetchDynamicDataItem(mailbox);

      return fetchDataItem;
    }

    private void EnsureDynamicAttributesFetched()
    {
      if (dynamicAttribute != null)
        return;

      Client.RunExclusively(() => FetchDynamicAttributes());
    }

    private void FetchDynamicAttributes()
    {
#if DEBUG
      Client.ThrowIfNotInExclusiveOperation();
#endif

      CheckMessageNumberAvailable(SequenceOrUidSet);

      ImapMessageDynamicAttribute[] dynamicAttrs;

      Mailbox.ProcessMessageStatusChangedResult(Client.Session.Fetch(SequenceOrUidSet,
                                                                     GetFetchDynamicDataItem(Mailbox),
                                                                     out dynamicAttrs),
                                                dynamicAttrs);

      var dynamicAttr = dynamicAttrs.FirstOrDefault(FindBySequence);

      if (dynamicAttr == null) {
        SetDeleted();
        throw new ImapMessageVanishedException(this);
      }
      else {
        dynamicAttribute = dynamicAttr;
      }
    }

    public void ToggleFlags(ImapMessageFlag flag,
                            params ImapMessageFlag[] flags)
    {
      ToggleFlags(new ImapMessageFlagSet(flags.Prepend(flag)));
    }

    public void ToggleKeywords(string keyword,
                               params string[] keywords)
    {
      ToggleFlags(new ImapMessageFlagSet(keywords.Prepend(keyword)));
    }

    public void ToggleFlags(IImapMessageFlagSet flagsAndKeywords)
    {
      if (flagsAndKeywords == null)
        throw new ArgumentNullException("flagsAndKeywords");

      Client.RunExclusively(() => ToggleFlagsCore(flagsAndKeywords));
    }

#if ASYNC_OPS
    public Task ToggleFlagsAsync(ImapMessageFlag flag,
                                 params ImapMessageFlag[] flags)
    {
      return ToggleFlagsAsync(new ImapMessageFlagSet(flags.Prepend(flag)));
    }

    public Task ToggleKeywordsAsync(string keyword,
                                    params string[] keywords)
    {
      return ToggleFlagsAsync(new ImapMessageFlagSet(keywords.Prepend(keyword)));
    }

    public Task ToggleFlagsAsync(IImapMessageFlagSet flagsAndKeywords)
    {
      if (flagsAndKeywords == null)
        throw new ArgumentNullException("flagsAndKeywords");

      return Client.StartTaskExclusively(() => ToggleFlagsCore(flagsAndKeywords),
                                         CancellationToken.None);
    }
#endif

    private void ToggleFlagsCore(IImapMessageFlagSet flagsAndKeywords)
    {
      var addFlags = new ImapMessageFlagSet();
      var removeFlags = new ImapMessageFlagSet();

      if (dynamicAttribute == null)
        FetchDynamicAttributes();

      foreach (var flag in flagsAndKeywords) {
        if (dynamicAttribute.Flags.Contains(flag))
          removeFlags.Add(flag);
        else
          addFlags.Add(flag);
      }

      if (0 < removeFlags.Count)
        StoreCore(SequenceOrUidSet, ImapStoreDataItem.RemoveFlags(removeFlags));

      if (0 < addFlags.Count)
        StoreCore(SequenceOrUidSet, ImapStoreDataItem.AddFlags(addFlags));
    }
  }
}
